/**
 * @class Sb.SiteView
 * Listens the model for changes and renders site
*/
Sb.SiteView = function() {
	this.container = Ext.getBody().insertFirst({
		cls: 'sb-frame'
	});
	this.container.on('scroll', this.reanchorEditPanel);
	this.container.on('contextmenu', Sb.Editor.Ui.onContextMenu, Sb.Editor.Ui, {
		type: 'site'
	});
	Ext.EventManager.onWindowResize(this.reanchorEditPanel, this);
	this.backgroundImage = Ext.getBody().insertFirst({
		tag: 'img',
		id: 'site-background-image',
		src: Sb.Config.blankImageUrl,
		alt: '',
		cls: 'background-image'
	});
	Sb.Editor.model.on('load', this.onModelLoad, this);
	Sb.TopMessageBox.on('resize', function(mb, w, h) {
		this.container.setTop(h);
		this.backgroundImage.setTop(h);
	}, this);
};

Sb.SiteView.prototype = {

	/**
	 * Make edit panel always in one place, where it should be,
	 * for example under widget.
	 *
	 * We need this as our current implementation of superpanels emulates
	 * body as div with scroll.
	 */
	reanchorEditPanel: function() {
		if (!Sb.Editor.Ui.editComponent) {
			return;
		}
		var editPanel = Ext.ComponentMgr.types[Sb.Editor.Ui.editComponent.getXType()].editPanel;
		if (editPanel && editPanel.component) {
			editPanel.reanchor();
		}
	},

	/**
	 * Highlits possible drop-place in containers
	 */
	ddSpacer: {
		node: null,
		html: '<div class="sb-widget-dd-spacer"></div>',

		/**
		 * Inserts spacer in position relative contextEl
		 * @param {String} insertPlace Where to insert space - beforeBegin, afterBegin, beforeEnd, afterEnd
		 * @param {HTMLElement} contextEl
		 */
		insert: function(insertPlace, contextEl) {
			this.removeNode();
			this.node = Ext.DomHelper.insertHtml(insertPlace, contextEl, this.html);
		},

		/**
		 * Empty this ddSpacer object
		 */
		remove: function() {
			this.removeNode();
			delete this.position;
		},

		/**
		 * Removes ddSpacer html-node from dom
		 */
		removeNode: function() {
			if (this.node) {
				Ext.removeNode(this.node);
				delete this.node;
			}
		}
	},

	/**
	 * Fires when widget drag start
	 */
	onWidgetStartDrag: function() {
		Sb.WidgetTypeManager.get('navigation').cache.each(function(model) {
			Ext.getCmp('sb-widget-' + model.data.uuid).contentElement.mask();
		});
	},

	/**
	 * Fires when widget drag end
	 */
	onWidgetEndDrag: function() {
		Sb.WidgetTypeManager.get('navigation').cache.each(function(model) {
			var widget = Ext.getCmp('sb-widget-' + model.data.uuid);
			if (widget) {
				widget.contentElement.unmask();
			}
		});
	},

	/**
	 * Initial actions after model loaded
	 */
	onModelLoad: function() {
		Sb.Editor.model.design.on('colorschemechange', this.onColorSchemeChange, this);
		Sb.Editor.model.design.on('decorationchange', this.onDecorationChange, this);
		Sb.Editor.model.design.on('fontchange', this.onFontChange, this);
		Sb.Editor.model.design.on('fontcolorchange', this.onFontColorChange, this);
		Sb.Editor.model.design.on('backgroundcolorchange', this.onBackgroundColorChange, this);

		this.updateCssFiles();
		this.onColorSchemeChange();
		this.onBackgroundColorChange();
		this.onFontChange();
		this.onFontColorChange();

		this.startPage = Sb.Editor.model.rootPage.pages.first();
		this.setCurrentPage(this.startPage);

		this._applyBgImages();
	},

	/**
	 * Change current site page to another specified
	 * @param {Sb.model.SitePage} page
	 */
	setCurrentPage: function(page) {
		this.currentPage = page;
		if (Sb.Editor.Ui.editComponent) {
			Sb.Editor.Ui.editComponent.completeEdit();
		}
		this.compilePage(page);
	},

	/**
	 * Compiles whole page
	 * @param {Sb.model.SitePage} page Site page model node
	 */
	compilePage: function(page) {
		if (this.pageWidget) {
			this.pageWidget.destroy();
		}

		this.pageWidget = Ext.create({
			xtype : page.widget.data.type + '_widget',
			model : page.widget,
			renderTo: this.container
		});

		// fix internal site links
		this.container.select('a[href="./"]').each(function(link) {
			this.wrapLink(link, this.startPage);
		}, this);

		Sb.Editor.model.rootPage.pages.each(function(page) {
			this.container.select('a[href$="' + page.getFileName() + '"]').each(function(link) {
				this.wrapLink(link, page);
			}, this);

			page.pages.each(function(subPage) {
				this.container.select('a[href$="' + subPage.getFileName() + '"]').each(function(link) {
					this.wrapLink(link, subPage);
				}, this);
			}, this);
		}, this);
	},

	/**
	 * Wrap link and adds click-event handler
	 * @param {Ext.Element} link
	 * @param {Sb.model.SitePage} page Site page model node
	 * @return none
	 */
	wrapLink: function(link, page) {
		link.dom.href = "#" + escape(page.getFileName());
		link.on('click', function(event, el, options) {
			event.stopEvent();
			this.setCurrentPage(options.page);
		}, this, {'page' : page});
	},

	/**
	 * Attach (replace) CSS files of template and decorations
	 */
	updateCssFiles: function() {
		var site			= Sb.Editor.model,
			designSettings	= Sb.Editor.model.design.getSettings(),
			config			= Sb.Config,
			version			= config.version + '_' + config.build,
			templateUrl		= config.templatesUrl + '/' + encodeURIComponent(site.data.template.code);

		Ext.util.CSS.swapStyleSheet('template-css', templateUrl + '/css/style.css?' + version);

		if (Ext.isIE) {
			Ext.util.CSS.swapStyleSheet('template-css-ie', templateUrl + '/css/ie.css?' + version);
		}

		if (Ext.isIE7) {
			Ext.util.CSS.swapStyleSheet('template-css-ie7', templateUrl + '/css/ie7.css?' + version);
		}

		Ext.each(site.design.getDecoratorList('background'), function(decoration) {
			this._attachDecorationCss('background', decoration);
		}, this);

		var backgroundUrl = templateUrl + '/decorations/background/';
		var filtersCss = '';
		for (var i=2; i <= 25; i++) {
			filtersCss = filtersCss +
			'.background' + i + '-black {filter: progid:DXImageTransform.Microsoft.AlphaImageLoader('+ 'src="' + backgroundUrl
			+ 'background' + i + '-black/images/background' + i + '-black.png", sizingmethod=scale, enabled=true); }'
			+ '.background' + i + '-white {filter: progid:DXImageTransform.Microsoft.AlphaImageLoader('+ 'src="' + backgroundUrl
			+ 'background' + i + '-white/images/background' + i + '-white.png", sizingmethod=scale, enabled=true); }';
		}
		Sb.StyleManager.addStyleSheet(filtersCss, 'ie-filters');
	},

	/**
	 * Attach css file for decoration
	 *
	 * @param {String} decorationType
	 * @param {String} decoration
	 */
	_attachDecorationCss: function(decorationType, decoration) {
		var decorationName	= 'template-decoration-' + decorationType + '-' + decoration + '-css',
			config			= Sb.Config,
			version			= config.version + '_' + config.build;

		Ext.util.CSS.swapStyleSheet(
			decorationName,
			config.templatesUrl + '/'
			+ Sb.Editor.model.data.template.code + '/decorations/'
			+ decorationType + '/'
			+ decoration + '/css/style.css?' + version
		);
	},

	/**
	 * Applying gradient images for backgrounds
	 */
	_applyBgImages: function() {
		var backgroundPath = Sb.Config.templatesUrl + '/' + Sb.Editor.model.data.template.code + '/decorations/background/';
		var containers = ['site', 'watermark', 'header', 'footer', 'content', 'column1', 'column2'];
		var designSettings = Sb.Editor.model.design.getSettings();
		Ext.each(containers, function(container) {
			var decorator = designSettings.background[container + 'Background'];
			Ext.get(container + '-background-image').dom.setAttribute('src',
				backgroundPath + decorator + '/images/' + decorator + '.png');
		});
	},

	/**
	 * Updates decorations
	 */
	onDecorationChange: function(decorationType, decoration) {
		if (decorationType != 'external-border' && decorationType != 'border') {
			this._attachDecorationCss(decorationType, decoration);
		}
		this._applyBgImages();
	},

	/**
	 * Update color scheme css
	 */
	onColorSchemeChange: function() {
		var design				= Sb.Editor.model.design,
			palette				= design.palette,
			colorScheme			= design.data.colorScheme,
			regions				= ['content', 'header', 'footer', 'column1', 'column2'],
			css					= '';

		Ext.each(regions, function(region) {
			css += Sb.WidgetTypeManager.get('navigation').getColorSchemeCssTemplate().apply({
				className						: '.' + region + ' .navigation',
				level1ContainerBackgroundColor	: palette[region + 'NavigationLevel1ContainerBackgroundColor'],
				level1Color						: palette[region + 'NavigationLevel1Color'],
				level1BackgroundColor			: palette[region + 'NavigationLevel1BackgroundColor'],
				level1ColorSelected				: palette[region + 'NavigationLevel1ColorSelected'],
				level1BackgroundColorSelected	: palette[region + 'NavigationLevel1BackgroundColorSelected'],
				level1ColorHovered				: palette[region + 'NavigationLevel1ColorHovered'],
				level1BackgroundColorHovered	: palette[region + 'NavigationLevel1BackgroundColorHovered'],
				level2ContainerBackgroundColor	: palette[region + 'NavigationLevel2ContainerBackgroundColor'],
				level2Color						: palette[region + 'NavigationLevel2Color'],
				level2BackgroundColor			: palette[region + 'NavigationLevel2BackgroundColor'],
				level2ColorSelected				: palette[region + 'NavigationLevel2ColorSelected'],
				level2BackgroundColorSelected	: palette[region + 'NavigationLevel2BackgroundColorSelected'],
				level2ColorHovered				: palette[region + 'NavigationLevel2ColorHovered'],
				level2BackgroundColorHovered	: palette[region + 'NavigationLevel2BackgroundColorHovered']
			});
			css += Sb.WidgetTypeManager.get('breadcrumbs').getColorSchemeCssTemplate().apply({
				className: '.' + region + ' .sb-widget-breadcrumbs .sb-widget-content',
				linkColor: palette[region + 'NavigationLevel1Color']
			});
		});
		css += Sb.WidgetTypeManager.get('search').getColorSchemeCssTemplate().apply({
			globalItemBorderColor	: colorScheme.color3,
			resultsUrlColor			: colorScheme.color3
		});

		Sb.StyleManager.addStyleSheet(css, 'colorscheme');
	},

	/**
	 * Update css for fonts
	 */
	onFontChange: function() {
		var fontData = Ext.apply({}, Sb.Editor.model.design.getSettings().font);
		if (!Sb.SiteView.fontTemplate) {
			Sb.SiteView.fontTemplate = new Ext.XTemplate(
				'.pageContentText{',
					'font-family:\'{textFont}\';',
					'font-size:{textFontSize}px;',
					'font-weight:{[values.textFontBold == "1" ? "bold" : "normal"]};',
					'font-style:{[values.textFontItalic == "1" ? "italic" : "normal"]};',
				'}',
				'.pageContentText p {',
					'text-decoration:{[values.textFontUnderline == "1" ? "underline" : "none"]};',
				'}',
				'.pageContentText a{',
					'font-family:\'{textFont}\';',
					'font-size:{textFontSize}px;',
					'font-weight:{[values.linkFontBold == "1" ? "bold" : "normal"]};',
					'font-style:{[values.linkFontItalic == "1" ? "italic" : "normal"]};',
					'text-decoration:{[values.linkFontUnderline == "1" ? "underline" : "none"]};',
				'}',
				'.pageContentText h1{',
					'font-family:\'{h1Font}\';',
					'font-size:{h1FontSize}px;',
					'font-weight:{[values.h1FontBold == "1" ? "bold" : "normal"]};',
					'font-style:{[values.h1FontItalic == "1" ? "italic" : "normal"]};',
					'text-decoration:{[values.h1FontUnderline == "1" ? "underline" : "none"]};',
				'}',
				'.pageContentText h2{',
					'font-family:\'{h2Font}\';',
					'font-size:{h2FontSize}px;',
					'font-weight:{[values.h2FontBold == "1" ? "bold" : "normal"]};',
					'font-style:{[values.h2FontItalic == "1" ? "italic" : "normal"]};',
					'text-decoration:{[values.h2FontUnderline == "1" ? "underline" : "none"]};',
				'}'
			);
		}

		Sb.StyleManager.addStyleSheet(
			Sb.SiteView.fontTemplate.apply(fontData),
 			'font-set'
		);
	},

	/**
	 * Update css for fonts colors
	 */
	onFontColorChange: function() {
		if (!Sb.SiteView.fontColorCssTemplate) {
			Sb.SiteView.fontColorCssTemplate = new Ext.XTemplate(
				'{className} { color: {textColor}; }',
				'{className} a { color: {linkColor}; }',
				'{className} a.link { color: {linkColor} !important; }',
				'{className} h1 { color: {h1Color}; }',
				'{className} h2 { color: {h2Color}; }'
			);
		}

		var design			= Sb.Editor.model.design,
			fontColor		= design.data.fontColor,
			regions			= ['content', 'header', 'footer', 'column1', 'column2'],
			css				= '';

		Ext.each(regions, function(region) {
			css += Sb.SiteView.fontColorCssTemplate.apply({
				className: '.' + region + '-color',
				textColor: Sb.Editor.model.design.data.fontColor[region + 'TextColor'],
				linkColor: Sb.Editor.model.design.data.fontColor[region + 'LinkColor'],
				h1Color: Sb.Editor.model.design.data.fontColor[region + 'H1Color'],
				h2Color: Sb.Editor.model.design.data.fontColor[region + 'H2Color']
			});
		});
		css += Sb.WidgetTypeManager.get('search').getFontColorCssTemplate().apply({
			resultsHeaderLinkColor	: fontColor.contentLinkColor
		});

		Sb.StyleManager.addStyleSheet(css, 'text-color');
	},

	/**
	 * Update site background color.
	 */
	onBackgroundColorChange: function() {
		Ext.getBody().setStyle(
			'background-color',
			Sb.Editor.model.design.data.backgroundColor.siteColor
		);
	}
};