/**
 * @class Sb.editor.PagesPanel
 * @extends Sb.editor.BaseFormPanel
 * Site pages panel
 * @constructor
 * @param {Object} config The config object
 * @xtype pagespanel
 */
Sb.editor.PagesPanel = Ext.extend(Sb.editor.BaseFormPanel, {

	// private
	initComponent: function() {
		this.treeSelectionModel = new Ext.tree.DefaultSelectionModel({
			listeners: {
				scope: this,
				selectionchange: function(sm, node) {
					if (!node) {
						return;
					}
					var model = node.attributes.model;
					this.protectedField.suspendEvents();
					this.getForm().setValues(Ext.apply({
							'protected' : model.isProtected()
						},
						model.data
					));
					this.protectedField.resumeEvents();
				}
			}
		});

		Ext.apply(this, {
			cls: 'sb-editor-pages-panel',
			title: SbGetLocaleByKey('pagesTitle'),
			helperMessage: SbGetLocaleByKey('pagesDescription'),
			tabTip: SbGetLocaleByKey('pagesDescriptionOnHover'),
			labelAlign: 'top',
			layout: 'column',
			autoHeight: true,
			defaults: {
				border: false,
				buttonAlign: 'left'
			},
			items: [{
				columnWidth: 0.6,
				ref: 'pagesTree',
				xtype: 'treepanel',
				baseCls: 'sb-pages-tree',
				autoScroll: true,
				animate: true,
				containerScroll: true,
				enableDD: true,
				lines: false,
				loader: new Ext.tree.TreeLoader({
					doPreload: function(node) {
						node.beginUpdate();
						node.attributes.model.pages.each(function(page) {
							var data = page.data,
								cls = [];
							if (data.hidden) {
								cls.push('sb-pages-tree-node-hidden');
							}
							if (page.isProtected()) {
								cls.push('sb-pages-tree-node-protected');
							}
							var attr = {
									expanded	: true,
									id			: data.identity,
									text		: data.title,
									iconCls		: 'sb-text-icon',
									cls			: cls.join(' '),
									model		: page,
									maxDepth	: 3,
									uiProvider	: Sb.editor.PagesTreeNodeUI
								},
								isEmpty	= page.pages.getCount() == 0,
								newNode = isEmpty ? new Ext.tree.TreeNode(attr) : new Ext.tree.AsyncTreeNode(attr);

							node.appendChild(newNode);
						});
						node.endUpdate();
						return true;
					}
				}),
				rootVisible: false,
				root: {
					id: Sb.Editor.model.rootPage.data.identity,
					model: Sb.Editor.model.rootPage,
					loaded: true
				},
				selModel: this.treeSelectionModel,
				listeners: {
					movenode: function(tree, node, oldParent, newParent, index) {
						newParent.attributes.model.insert(
							index,
							node.attributes.model,
							oldParent.attributes.model
						);
						Sb.editor.UiContext.registerChanges();
						node.select();
					},
					startdrag: function(panel, node, e) {
						var el = panel.dragZone.proxy.el;
						el.disableShadow();
						el.setXY(el.getXY());
						var ghost = panel.dragZone.proxy.ghost;
						ghost.addClass('sb-pages-tree-drag-ghost');
						ghost.setWidth(Ext.get(node.ui.wrap).getWidth());
					},
					beforemovenode: function(tree, node, oldParent, newParent, index) {
						return node.getUI().allowMoveToNode(newParent);
					}
				}
			}, {
				columnWidth: 0.4,
				layout: 'form',
				baseCls: 'sb-pages-page-properties',
				layoutConfig: {
					labelSeparator: ''
				},
				defaults: {
					validateOnBlur: false
				},
				items: [{
					ref: '../titleField',
					xtype: 'textfield',
					fieldLabel: SbGetLocaleByKey('pageTitle'),
					name: 'title',
					maxLength: 255,
					allowBlank: false,
					anchor: '100%',
					msgTarget: 'under',
					listeners: {
						valid: this.onChange.createDelegate(this),
						focus: function() {
							Sb.Editor.Ui.mainWindow.setHelperMessage(SbGetLocaleByKey('messagePageTitle'));
						}
					}
				}, {
					ref: '../browserTitleField',
					xtype: 'textarea',
					fieldLabel: SbGetLocaleByKey('pageBrowserTitle'),
					name: 'browserTitle',
					maxLength: 255,
					anchor: '100%',
					style: 'height: 34px',
					msgTarget: 'under',
					listeners: {
						valid: this.onChange.createDelegate(this),
						focus: function(field) {
							if (field.getValue() == '') {
								field.setRawValue(field.emptyText);
							}
							Sb.Editor.Ui.mainWindow.setHelperMessage(SbGetLocaleByKey('messagePageBrowserTitle'));
						}
					}
				},{
					ref: '../descriptionField',
					xtype: 'textarea',
					fieldLabel: SbGetLocaleByKey('pageDescription'),
					name: 'description',
					msgTarget: 'under',
					maxLength: 255,
					anchor: '100%',
					style: 'height: 34px',
					listeners: {
						valid: this.onChange.createDelegate(this),
						focus: function() {
							Sb.Editor.Ui.mainWindow.setHelperMessage(SbGetLocaleByKey('messageMetaDescription'));
						}
					}
				},{
					ref: '../keywordsField',
					xtype: 'textarea',
					fieldLabel: SbGetLocaleByKey('pageKeywords'),
					name: 'keywords',
					msgTarget: 'under',
					maxLength: 255,
					anchor: '100%',
					style: 'height: 34px',
					listeners: {
						valid: this.onChange.createDelegate(this),
						focus: function() {
							Sb.Editor.Ui.mainWindow.setHelperMessage(SbGetLocaleByKey('messageMetaKeywords'));
						}
					}
				}, {
					ref: '../hiddenField',
					xtype: 'checkbox',
					name: 'hidden',
					hideLabel: true,
					boxLabel: SbGetLocaleByKey('pageHidden'),
					listeners: {
						check: this.onChange.createDelegate(this),
						afterrender: function() {
							var fn = Sb.Editor.Ui.mainWindow.setHelperMessage.createDelegate(
								Sb.Editor.Ui.mainWindow, [SbGetLocaleByKey('hiddenPageTooltip')]
							);
							this.el.on('mouseenter', fn);
							this.el.on('mouseleave', Sb.Editor.Ui.mainWindow.restoreHelperMessage, Sb.Editor.Ui.mainWindow);

							this.labelEl.on('mouseenter', fn);
							this.labelEl.on('mouseleave', Sb.Editor.Ui.mainWindow.restoreHelperMessage, Sb.Editor.Ui.mainWindow);
						}
					}
				}, {
					ref			: '../protectedField',
					xtype		: 'checkbox',
					name		: 'protected',
					itemCls		: 'sb-pages-protected-field',
					hideLabel	: true,
					boxLabel	: [
						SbGetLocaleByKey('passwordProtectPageTitle'), '<a href="#setCredentials">',
						Ext.util.Format.htmlEncode(SbGetLocaleByKey('passwordProtectPageSetCredentials')), '</a>'
					].join(' '),
					listeners	: {
						check		: this.protectPage.createDelegate(this),
						afterrender	: function(fld) {
							var fn = Sb.Editor.Ui.mainWindow.setHelperMessage.createDelegate(
								Sb.Editor.Ui.mainWindow, [SbGetLocaleByKey('passwordProtectPageDescription')]
							);
							fld.el.on('mouseenter', fn);
							fld.el.on('mouseleave', Sb.Editor.Ui.mainWindow.restoreHelperMessage, Sb.Editor.Ui.mainWindow);

							fld.labelEl.on('mouseenter', fn);
							fld.labelEl.on('mouseleave', Sb.Editor.Ui.mainWindow.restoreHelperMessage, Sb.Editor.Ui.mainWindow);
							fld.labelEl.child('a').on('click', function (e) {
								e.stopEvent();
								this.protectPage(fld);
							}, this);
						}.createDelegate(this)
					}
				}]
			}, {
				columnWidth: 0.6,
				layout: 'form',
				baseCls: 'sb-pages-page-buttons',
				buttons: [{
					text: SbGetLocaleByKey('addPage'),
					tooltip: SbGetLocaleByKey('addPageTooltip'),
					iconCls: 'sb-add-icon',
					handler : this.createPage.createDelegate(this, ['text'])
				}, {
					text: SbGetLocaleByKey('deletePage'),
					tooltip: SbGetLocaleByKey('deletePageTooltip'),
					iconCls: 'sb-delete-icon',
					handler : this.deletePage.createDelegate(this)
				}]
			}, {
				columnWidth: 0.4,
				layout: 'form',
				baseCls: 'sb-pages-page-buttons',
				buttons: [{
					text: SbGetLocaleByKey('openPage'),
					tooltip: SbGetLocaleByKey('openPageTooltip'),
					iconCls: 'sb-open-page-icon',
					scope: this,
					handler: function() {
						Sb.Editor.siteView.setCurrentPage(
							this.treeSelectionModel.getSelectedNode().attributes.model
						);
					}
				}]
			}]
		});

		/**
		 * Object for saving last entered credentials for prefilling it later
		 * @property lastSavedCredentials
		 * @type Object
		 */
		this.lastSavedCredentials = {};

		Sb.editor.PagesPanel.superclass.initComponent.apply(this, arguments);
	},

	/**
	 * Activate handler
	 */
	onActivate: function() {
		this.memento = Sb.Editor.model.rootPage.saveToMemento();
		var rootNode = this.pagesTree.getRootNode();
		rootNode.reload();
		// select current page in tree
		rootNode.findChild('id', Sb.Editor.siteView.currentPage.data.identity, true).select();

		Sb.editor.PagesPanel.superclass.onActivate.apply(this, arguments);
	},

	/**
	 * Change handler
	 *
	 * @param {Ext.form.Field} field
	 */
	onChange: function(field) {
		var node = this.treeSelectionModel.getSelectedNode();
		switch (field.name) {
			case 'title':
				node.setText(Ext.util.Format.htmlEncode(
					Ext.util.Format.ellipsis(field.getValue(), 27 - node.getDepth() * 2)
				));
				if (this.browserTitleField.getValue() == '') {
					this.browserTitleField.setRawValue(field.getValue());
				}
				this.browserTitleField.emptyText = field.getValue();
				break;
			case 'hidden':
				node.ui[(field.getValue() ? 'add' : 'remove') + 'Class']('sb-pages-tree-node-hidden');
				break;
		}

		if (node.attributes.model.setProperty(field.name, field.getValue())) {
			Sb.editor.UiContext.registerChanges();
		}
	},

	/**
	 * Create page by type
	 * @param {String} type
	 */
	createPage: function(type) {
		var currentNode	= this.treeSelectionModel.getSelectedNode(),
			newPage		= currentNode.attributes.model.createPage(),
			newPageNode = null;

		switch (type) {
			case 'text':
				newPage.data.title = SbGetLocaleByKey('newTextPage');
				newPage.data.browserTitle = '';

				this.addTextWidget(newPage, 'content');
				this.addTextWidget(newPage, 'column1');
				this.addTextWidget(newPage, 'column2');

				newPageNode = new Ext.tree.TreeNode({
					id			: newPage.data.identity,
					text		: newPage.data.title,
					iconCls		: 'sb-text-icon',
					model		: newPage,
					maxDepth	: 3,
					uiProvider	: Sb.editor.PagesTreeNodeUI
				});
				break;
		}

		currentNode.parentNode.insertBefore(newPageNode, currentNode.nextSibling);
		currentNode.parentNode.attributes.model.insertChild(newPage, currentNode.parentNode.indexOf(newPageNode));
		Sb.editor.UiContext.registerChanges();
		newPageNode.select();
	},

	/**
	 * Add new text widget with empty content
	 * @param {Sb.model.SitePage} page
	 * @param {String} region
	 */
	addTextWidget: function(page, region) {
		var widgetContainer = page.widget.widgets.find(function(container) {
			return container.data.region == region;
		});

		var contentWidget = new Sb.model.Widget({
			type: 'text',
			content: ''
		});

		widgetContainer.widgets.insert(0, contentWidget);
	},

	/**
	 * Delete page from model and tree
	 * @return {Boolean}
	 */
	deletePage: function() {
		var rootNode = this.pagesTree.getRootNode();
		var nodeForDelete = this.treeSelectionModel.getSelectedNode();
		if (!nodeForDelete) {
			return false;
		}
		if ((nodeForDelete.parentNode === rootNode) && (rootNode.childNodes.length == 1)) {
			Sb.Editor.Ui.mainWindow.statusBar.setMessageInfo(SbGetLocaleByKey('cannotRemoveAllPages'));
			return false;
		}

		var nodeForSelect = nodeForDelete.nextSibling || nodeForDelete.previousSibling || nodeForDelete.parentNode;

		//:INFO: workaround for bug #44214
		var c = this.pagesTree.body.dom;
		var offset = c.scrollHeight - c.scrollTop - c.clientHeight - nodeForDelete.ui.wrap.clientHeight;
		if (offset < 0) {
			c.scrollTop += offset;
		}

		nodeForDelete.parentNode.attributes.model.remove(nodeForDelete.attributes.model);
		nodeForDelete.parentNode.removeChild(nodeForDelete);

		nodeForDelete.attributes.model.applySelfAndRecursive(function (page) {
			if (page.data.identity == Sb.Editor.siteView.currentPage.data.identity) {
				Sb.Editor.siteView.setCurrentPage(
					nodeForSelect.attributes.model
				);
			}
		});
		nodeForSelect.select();

		Sb.editor.UiContext.registerChanges();
		return true;
	},

	/**
	 * Open protect page dialog or unprotect page
	 *
	 * @param {Ext.form.Field} field
	 */
	protectPage: function(field) {
		var node		= this.treeSelectionModel.getSelectedNode(),
			page		= node.attributes.model,
			data		= page.data,
			saveHandler = function(dlg) {
				var credentials	= {
						login		: dlg ? dlg.loginField.getValue() : null,
						password	: dlg ? dlg.passwordField.getValue() : null
					},
					isSaved = page.unserialize(credentials);
				if (isSaved) {
					node.ui[(page.isProtected() ? 'add' : 'remove') + 'Class']('sb-pages-tree-node-protected');
					Sb.editor.UiContext.registerChanges();
					if (credentials.login) {
						this.lastSavedCredentials = credentials;
					}
				}
			}.createDelegate(this);

		if (!field.getValue()) {
			saveHandler();
			return;
		}

		(new Sb.editor.PageProtectDialog({
			login			: data.login || this.lastSavedCredentials.login,
			password		: data.password || this.lastSavedCredentials.password,
			okHandler		: saveHandler,
			cancelHandler	: function() {
				if (field.getValue() != page.isProtected()) {
					field.setValue(page.isProtected());
				}
			}
		})).show();
	},

	/**
	 * OK button handler.
	 */
	okHandler: function() {
		if (!this.form.isValid()) {
			Sb.Editor.Ui.mainWindow.statusBar.setMessageError(
				SbGetLocaleByKey('fillRequiredFields')
			);
			return;
		}
		Sb.editor.PagesPanel.superclass.okHandler.call(this);
	},

	/**
	 * Checks that at least 1 page is visible
	 */
	isAllPagesHidden: function() {
		var firstVisibleNode = this.pagesTree.root.findChildBy(
			function(child) {
				return !child.attributes.model.data.hidden;
			},
			this
		);
		return !firstVisibleNode;
	},

	/**
	 * Save handler.
	 * @param {Function} afterSaveCallbacks Handler, that will be run after save
	 */
	saveHandler: function(afterSaveCallbacks) {
		if (this.isAllPagesHidden()) {
			Sb.Editor.Ui.mainWindow.statusBar.setMessageError(
				SbGetLocaleByKey('allPagesHidden')
			);
			afterSaveCallbacks.failure.call();
			return;
		}

		Sb.Editor.model.rootPage.save({
			success: function() {
				afterSaveCallbacks.success.call();
				Sb.Editor.Ui.mainWindow.statusBar.setMessage(
					SbGetLocaleByKey('messageSitemapChangesSaved')
				);
			},
			failure: afterSaveCallbacks.failure
		});
	},

	/**
	 * Discard handler
	 */
	discardHandler: function() {
		Sb.Editor.model.rootPage.restoreFromMemento(this.memento);
	}
});
Ext.reg('pagespanel', Sb.editor.PagesPanel);