/**
 * @class Sb.editor.component.Base
 * @extends Ext.Component
 * Base class for sitebuilder component - hovering, drag and drop, clicking
 * @constructor
 * @param {Object} config The config object
 */
Sb.editor.component.Base = Ext.extend(Ext.Component, {
	/**
	 * @cfg {String} baseCls
	 * The base CSS class to apply to this component's element (defaults to <code>'sb-component'</code>).
	 */
	baseCls: 'sb-widget',

	/**
	 * @cfg {Boolean} hasEditMenu
	 * Is needed to create menu for Edit button or not (defaults to <code>true</code>).
	 */
	hasEditMenu: false,

	/**
	 * @cfg {Boolean} removable
	 * <p>True to display the 'Remove' button and allow to remove the component, false to hide
	 * the button and disallow removing the component (defaults to <code>true</code>).</p>
	 */
	removable: true,

	/**
	 * @cfg {Boolean} removeConfirmation
	 * True to remove the component with confirmation
	 */
	removeConfirmation: true,

	initComponent: function() {
		Sb.editor.component.Base.superclass.initComponent.apply(this, arguments);
		this.removeTitle = SbGetLocaleByKey('removeWidgetTitle');
		this.addEvents('startover', 'removeover', 'startedit', 'completeedit', 'canceledit', 'remove');
	},

	onRender: function() {
		this.autoEl = {
			cls: this.baseCls,
			cn: {
				cls: this.baseCls + '-inner',
				cn: [{
					cls: this.baseCls + '-controls',
					cn: {
						cls: this.baseCls + '-controls-wrap',
						cn: {
							cls: this.baseCls + '-controls-inner'
						}
					}
				}, {
					cls: this.baseCls + '-content'
				}]
			}
		};

		Sb.editor.component.Base.superclass.onRender.apply(this, arguments);

		this.innerEl = this.el.first();
		this.controlsEl = this.innerEl.first();
		this.controlsEl.setVisibilityMode(Ext.Element.DISPLAY);
		this.contentElement = this.controlsEl.next();
		this.controlsTargetEl = this.controlsEl.first().first();
		this.controlsEl.hide();

		this.ddEl = this.controlsTargetEl.createChild({
			tag: 'span',
			cls: this.baseCls + '-control-item ' + this.baseCls + '-control-move',
			cn: {
				tag: 'span'
			}
		});

		//hasEditMenu
		var editElCls = this.baseCls + '-control-item ' + this.baseCls + '-control-edit';
		if (!this.hasEditMenu) {
			editElCls += ' sb-widget-control-edit-no-menu';
		}
		this.editEl = this.controlsTargetEl.createChild({
			tag: 'span',
			cls: editElCls
		});
		this.editEl.button = this.editEl.createChild({
			tag: 'span',
			html: SbGetLocaleByKey('editWidgetTitle')
		});
		this.editEl.arrow = this.editEl.createChild({
			tag: 'i',
			cls: this.baseCls + '-control-dd-arrow'
		});

		if (this.removable) {
			this.removeEl = this.controlsTargetEl.createChild({
				tag: 'span',
				cls: this.baseCls + '-control-item ' + this.baseCls + '-control-remove',
				cn: {
					tag: 'span',
					html: this.removeTitle
				}
			});
		}

		this.applyEl = this.controlsEl.createChild({
			tag: 'span',
			cls: this.baseCls + '-control-btn ' + this.baseCls + '-control-btn-apply',
			style: 'display: none',
			cn: {
				tag: 'span',
				cls: this.baseCls + '-control-btn-inner',
				cn: {
					tag: 'span',
					html: SbGetLocaleByKey('applyChangesTitle')
				}
			}
		});
		this.applyEl.setVisibilityMode(Ext.Element.DISPLAY).hide();

		this.reverseEl = this.controlsEl.createChild({
			tag: 'span',
			cls: this.baseCls + '-control-btn ' + this.baseCls + '-control-btn-reverse',
			style: 'display: none',
			cn: {
				tag: 'span',
				cls: this.baseCls + '-control-btn-inner',
				cn: {
					tag: 'span',
					html: SbGetLocaleByKey('reverseChangesTitle')
				}
			}
		});
		this.reverseEl.setVisibilityMode(Ext.Element.DISPLAY).hide();

		if (this.hasEditMenu) {
			this.menu = new Ext.menu.Menu({
				cls: 'sb-menu-widget-actions',
				shadow: false,
				items: [],
				listeners: {
					scope: this,
					mouseover: function(menu, e, menuItem) {
						if (menuItem.tooltip) {
							Sb.Editor.Ui.mainWindow.setHelperMessage(menuItem.tooltip);
						}
					},
					mouseout: function(menu, e, menuItem) {
						if (menuItem.tooltip) {
							Sb.Editor.Ui.mainWindow.restoreHelperMessage();
						}
					},
					show: function(menu) {
						this.editEl.arrow.ignoreNextClick = 0;
						this.editEl.arrow.ignoreNextHide = 0;
						Sb.Editor.Ui.isHoverDisabled = true;
					},
					hide: function(menu) {
						Sb.Editor.Ui.isHoverDisabled = false;
						this.editEl.arrow.ignoreNextClick = this.restoreActionsClick.defer(250, this);
						if (!this.editEl.arrow.ignoreNextHide) {
							this.removeOver.defer(250, this);
						}
					}
				}
			});
		}
	},

	restoreActionsClick: function() {
		this.editEl.arrow.ignoreNextClick = 0;
	},

	restoreActionsMouseDown: function() {
		this.editEl.arrow.ignoreNextHide = 0;
	},

	getContentTarget: function() {
		return this.contentElement;
	},

	afterRender: function() {
		Sb.editor.component.Base.superclass.afterRender.call(this);
		this.initEvents();
	},

	initEvents: function() {
		Sb.Editor.windowManager.register(this, 1);

		this.contentElement.on('click', function(event) {
			if (this.afterDD) {
				this.afterDD = false;
				return;
			}
			this.startEdit();
		}, this, {stopEvent: true});

		this.getEl().on('mouseenter', function(e) {
			e.stopPropagation();
			if (Sb.Editor.Ui.overComponent == this
				|| Sb.Editor.Ui.editComponent
				|| Sb.Editor.Ui.isDragActive()
				|| Sb.Editor.Ui.isHoverDisabled
			) {
				return;
			}
			if (!this.showOverTask) {
				this.showOverTask = new Ext.util.DelayedTask(this.startOver, this);
			}
			this.showOverTask.delay(200);
		}, this);

		this.getEl().on('mouseleave', this.completeOver, this);

		this.controlsEl.on('click', Ext.emptyFn, this, {stopEvent:true});

		this.editEl.arrow.on('mousedown', function() {
			this.editEl.arrow.ignoreNextHide = this.restoreActionsMouseDown.defer(250, this);
		}, this);
		this.editEl.arrow.on('click', function() {
			if (!this.editEl.arrow.ignoreNextClick && Sb.Editor.Ui.editComponent != this && this.rendered && this.menu) {
				this.menu.show(this.editEl);
			}
		}, this);
		this.editEl.button.on('click', this.startEdit, this);

		if (this.removable) {
			this.removeEl.on('click', this.removeComponent, this);
		}
		this.applyEl.on('click', this.completeEdit, this);
		this.reverseEl.on('click', this.cancelEdit, this);

		this.manager.sendToBack(this);

		this.initDD();
	},

	initDD: function() {
		this.dd = new Sb.editor.component.Base.DD(this);
	},

	removeComponent: function() {
		if (!this.removeConfirmation) {
			this.removeComponentHandler();
			return;
		}

		Ext.Msg.show({
			title    : SbGetLocaleByKey('confirmRemoval'),
			msg      : SbGetLocaleByKey('confirmRemovalWidget'),
			closable : false,
			buttons  : Ext.Msg.YESNO,
			scope    : this,
			fn       : function(btn) {
				if (btn == 'yes') {
					this.removeComponentHandler();
				}
			}
		});
	},

	removeComponentHandler: function() {
		if (Sb.Editor.Ui.editComponent == this) {
			this.hideEdit();
		}
		this.fireEvent('remove', this);
	},

	showControls: function() {
		this.controlsEl.show();
		this.anchorControls();
	},

	anchorControls: function() {
		var top = this.el.getTop() - Sb.Editor.siteView.container.getTop();
		var height = this.controlsEl.getHeight();
		var alignment = top > height ? 'bl-tl' : 'tl-bl';
		this.controlsEl.anchorTo(this.el, alignment);
	},

	completeOver: function() {
		if (this.showOverTask) {
			this.showOverTask.cancel();
		}

		if (!Sb.Editor.Ui.overComponent) {
			return;
		}

		if (Sb.Editor.Ui.overComponent.showOverTask) {
			Sb.Editor.Ui.overComponent.showOverTask.cancel();
		}
		Sb.Editor.Ui.overComponent.removeOver();
	},

	startOver: function() {
		if (Sb.Editor.Ui.overComponent) {
			if (Sb.Editor.Ui.overComponent == this) {
				return;
			}
			Sb.Editor.Ui.overComponent.completeOver();
		}
		Sb.Editor.Ui.overComponent = this;
		this.addClass(this.getStateCls('over'));
		this.showControls();
		this.fireEvent('startover', this);
		this.manager.bringToFront(this);
	},

	/**
	 * Starts the editing process and shows the editor.
	 */
	startEdit: function() {
		if (Sb.Editor.Ui.editComponent) {
			if (Sb.Editor.Ui.editComponent == this) {
				return;
			}
			Sb.Editor.Ui.editComponent.completeEdit();
		}
		this.startOver();
		Sb.Editor.Ui.editComponent = this;
		this.applyEl.show();
		this.reverseEl.show();
		this.editEl.addClass(this.baseCls + '-control-item-disabled');
		this.el.addClass(this.getStateCls('focus'));
		this.onStartEdit();
		Ext.getDoc().on('mousedown', this.onDocumentClick, this);
		this.fireEvent('startedit', this);
	},

	onStartEdit: function() {
		this.getEditPanel().show(this);
	},

	/**
	 * Ends the editing process, persists the changed value to the underlying field, and hides the editor.
	 */
	completeEdit: function() {
		if (Sb.Editor.Ui.editComponent != this) {
			return;
		}
		this.onCompleteEdit();
		this.hideEdit();
		this.fireEvent('completeedit', this);
	},

	onCompleteEdit: Ext.emptyFn,

	/**
	 * Cancels the editing process and hides the editor without persisting any changes.
	 */
	cancelEdit: function() {
		if (Sb.Editor.Ui.editComponent != this) {
			return;
		}
		this.onCancelEdit();
		this.hideEdit();
		this.fireEvent('canceledit', this);
	},

	onCancelEdit: Ext.emptyFn,

	/**
	 * Hides the editor.
	 */
	hideEdit: function() {
		Ext.getDoc().un('mousedown', this.onDocumentClick, this);
		this.onHideEdit();
		this.applyEl.hide();
		this.reverseEl.hide();
		this.controlsEl.hide();
		this.editEl.removeClass(this.baseCls + '-control-item-disabled');
		this.el.removeClass(this.getStateCls('focus'));
		this.el.removeClass(this.getStateCls('over'));
		Sb.Editor.Ui.editComponent = null;
		Sb.Editor.Ui.overComponent = null;
	},

	onHideEdit: function() {
		this.getEditPanel().hide();
	},

	/**
	 * Remove "over" state
	 */
	removeOver: function() {
		if ((!Ext.dd.DragDropMgr.dragCurrent || Ext.dd.DragDropMgr.dragCurrent.component != this)
				&& Sb.Editor.Ui.editComponent != this && (!this.menu || this.menu.hidden)) {
			Sb.Editor.Ui.overComponent = null;
			this.manager.sendToBack(this);
			this.controlsEl.hide();
			this.removeClass(this.getStateCls('over'));
			this.fireEvent('removeover', this);
		}
	},

	/**
	 * Return component edit panel.
	 *
	 * @return {Sb.editor.component.BaseEditPanel} editPanel
	 */
	getEditPanel: function() {
		if (!Sb.editor.component.Base.editPanel) {
			Sb.editor.component.Base.editPanel = new Sb.editor.component.BaseEditPanel();
		}
		return Sb.editor.component.Base.editPanel;
	},

	onDocumentClick: function(event) {
		var editPanel		= this.getEditPanel(),
			targetHasMask	= Ext.fly(event.target).hasClass('ext-el-mask');

		if (editPanel && event.within(editPanel.el)
			|| event.within(this.el)
			|| (this.menu && event.within(this.menu.el))
			|| event.target == Sb.Editor.Ui.mainWindow.header.dom
			|| event.getTarget('#sb-editor-minimize-btn')
			|| event.getTarget('#sb-editor-help-btn')
			|| targetHasMask
			|| event.getTarget('.x-window-dlg')
			|| (Ext.fly(event.target).hasClass('sb-frame') && ((event.xy[0] >= event.target.clientWidth) || (event.xy[1] >= event.target.clientHeight)))
			|| event.button != 0
			|| (Ext.isSafari && targetHasMask && (
					event.xy[0] > event.target.clientWidth
					|| event.xy[1] > event.target.clientHeight
					|| event.xy[0] < 0
					|| event.xy[1] < 0
				))
			|| event.getTarget('.x-combo-list')
			|| event.getTarget('.sb-color-dropdown')
		) {
			return;
		}
		this.completeEdit();
	},

	/**
	 * Gets class for some state
	 *
	 * @param {String} state
	 * @return {String} class
	 **/
	getStateCls: function(state) {
		return this.baseCls + '-' + state;
	},

	resetStyles: function() {
		this.getEl().setStyle({
			position	: 'relative',
			left		: 'auto',
			top			: 'auto',
			width		: '100%'
		});
		this.removeClass(this.getStateCls('over'));
	},

	// support manager interface
	setZIndex: function(zindex) {
		this.getEl().setStyle('z-index', zindex);
	},

	// support manager interface
	setActive: Ext.emptyFn,

	beforeDestroy: function() {
		Sb.editor.component.Base.superclass.beforeDestroy.call(this);
		if (Sb.Editor.Ui.editComponent == this) {
			this.hideEdit();
		}
		if (this.menu) {
			Ext.destroy(this.menu);
		}
		if (this.controlsEl) {
			this.controlsEl.removeAnchor();
		}
	},

	onDestroy: function() {
		Sb.editor.component.Base.superclass.onDestroy.call(this);

		if (this.showOverTask) {
			this.showOverTask.cancel();
		}

		this.manager.unregister(this);
		if (Sb.Editor.Ui.overComponent == this) {
			if (Sb.Editor.Ui.overComponent.showOverTask) {
				Sb.Editor.Ui.overComponent.showOverTask.cancel();
			}
			Sb.Editor.Ui.overComponent = null;
		}
		if (Sb.Editor.Ui.editComponent == this) {
			Sb.Editor.Ui.editComponent = null;
		}
	}
});

//private - custom Component DD implementation
Sb.editor.component.Base.DD = function(component, cfg) {
	this.component = component;
	cfg = Ext.apply({ddGroup: 'widgets'}, cfg);

	Sb.editor.component.Base.DD.superclass.constructor.call(this, this.component.el, cfg);
	this.setOuterHandleElId(this.component.ddEl);
	this.component.ddEl.setStyle('cursor', 'move');
	this.scroll = false;
};

Ext.extend(Sb.editor.component.Base.DD, Ext.dd.DragSource, {
	getDragEl: function() {
		return Ext.getDom(this.component.getEl());
	},

	b4MouseDown: function(e) {
		var x = e.getPageX();
		var y = e.getPageY();
		this.autoOffset(x, y);
	},

	autoOffset: function(x, y) {
		x -= this.startPageX;
		y -= this.startPageY;
		this.setDelta(x, y);
	},

	startDrag: function (x, y) {
		Sb.Editor.siteView.onWidgetStartDrag();

		if (Sb.Editor.Ui.editComponent) {
			Sb.Editor.Ui.editComponent.completeEdit();
			if (!this.component.controlsEl.isVisible()) {
				this.component.showControls();
			}
		}

		// :TODO: echernodarov 20100420: dirty hack, need rework
		this.component.afterDD = true;
		this.component.setZIndex(Sb.Editor.windowManager.dragZIndex);
		this.component.ownerCt && this.component.ownerCt.notifyOver(this, [x, y]);
		this.component.el.setStyle({
			position: 'absolute',
			width: this.component.el.getWidth() + 'px'
		});
		if (Ext.isIE7) {

			/**
			 * Component grand-parent container during DD
			 *
			 * @type Ext.Element
			 * @property ddComponentContainer
			 */
			this.ddComponentContainer = this.getComponentContainer();
			if (this.ddComponentContainer) {
				this.ddComponentContainer.setStyle('z-index', Sb.Editor.windowManager.dragZIndex);
			}
		}
	},

	getComponentContainer: function() {
		return this.component.el.up('.container-content-inner', 1).up('div', 1);
	},

	endDrag: function(e) {
		if (Ext.isIE7) {
			if (this.ddComponentContainer) {
				this.ddComponentContainer.setStyle('z-index', 0);
			}
			this.ddComponentContainer = null;
		}
		delete this.invalidDrop;
		Sb.Editor.siteView.onWidgetEndDrag();
	},

	onInvalidDrop: function () {
		this.invalidDrop = true;
		var animCfgObj = {
			easing		: 'elasticOut',
			duration	: 0.3,
			scope		: this,
			callback	: function() {
				Sb.Editor.siteView.ddSpacer.remove();
				this.component.resetStyles();
			}
		};

		this.el.moveTo(this.startPageX, this.startPageY, animCfgObj);
	},
	createFrame	: Ext.emptyFn,
	showFrame	: Ext.emptyFn
});