/**
 * @class Sb.editor.component.Text
 * @extends Sb.editor.component.Base
 * Class for text component
 * @constructor
 * @param {Object} config The config object
 * @xtype text_component
 */
Sb.editor.component.Text = Ext.extend(Sb.editor.component.Base, {
	removeConfirmation: false,

	/**
	 * @cfg {String} emptyText
	 * Fallback text when component's content is empty (defaults to <code>SbGetLocaleByKey('textComponentEmptyMessage')</code>).
	 */
	emptyText: SbGetLocaleByKey('textComponentEmptyMessage'),

	initComponent: function() {
		Sb.editor.component.Text.superclass.initComponent.apply(this, arguments);
		this.removeTitle = SbGetLocaleByKey('hideWidgetTitle');
		this.data = {};
	},

	getEditPanel: function() {
		if (!Sb.editor.component.Text.editPanel) {
			Sb.editor.component.Text.editPanel = new Sb.editor.component.TextEditPanel();
		}
		return Sb.editor.component.Text.editPanel;
	},

	onStartEdit: function() {
		Sb.editor.component.Text.superclass.onStartEdit.apply(this);
		this.memento = Ext.apply({}, this.data);
		this.contentElement.set({contentEditable: 'true'});
		if (this.contentElement.dom.innerHTML == this.emptyText) {
			this.contentElement.update('');
		}
		this.contentFocus();
		this.contentElement.on('keyup', this.onKeyUp, this);
	},

	onCompleteEdit: function() {
		this.setData({content: this.contentElement.dom.innerHTML});
		Sb.editor.component.Text.superclass.onCompleteEdit.apply(this);
	},

	onCancelEdit: function() {
		Sb.editor.component.Text.superclass.onCancelEdit.apply(this);
		this.setData(this.memento);
	},

	onHideEdit: function() {
		this.contentElement.set({contentEditable: 'false'});
		Sb.editor.component.Text.superclass.onHideEdit.apply(this);
	},

	onKeyUp: function(e) {
		this.data.content = this.contentElement.dom.innerHTML;
		if (this.contentElement.dom.innerHTML != '') {
			this.getEditPanel().reanchor();
		}
	},

	setContent: function(content) {
		var data		= this.data,
			stripTags	= Ext.util.Format.stripTags,
			trim		= function(text) {
				text.replace(/^(\&nbsp;)*||(\&nbsp;)*$/, '');
				return text;
			},
			hasVisibleSymbols	= function(text) {
				text = text.replace(/\&nbsp;/g, '');
				text = Ext.util.Format.stripTags(text);
				text = Ext.util.Format.trim(text);
				return (text != '');
			},
			saveTags	= '<br><br/>';

		content	= content	.replace(/<\/(p|div)>/gi,		'')			// Clean up all closing tags
							.replace(/^<(p|div)[^>]*>/i,	'')			// Clean up 1st opening tag
							.replace(/<(p|div)[^>]*>/gi,	'<br/>');	// Replace remaining opening tags

		content = stripTags(content, saveTags);
		content.replace(/(<br\/?>)*$/, '');								// Delete all <br> at the end

		content = (hasVisibleSymbols(content)) ? trim(content) : '';

		data.content = content;

		if (this.rendered) {
			var viewContent = (data.content != '') ? data.content : this.emptyText;
			this.contentElement.update(viewContent);
			if (Sb.Editor.Ui.editComponent && Sb.Editor.Ui.editComponent == this) {
				this.getEditPanel().reanchor();
			}
		}
	},

	setData: function(newData) {
		Ext.apply(this.data, newData);
		var data = this.data;
		if ('content' in newData) {
			this.setContent(newData.content);
		}

		if (this.rendered) {
			var style = {
				'color'				: data.fontColor,
				'font-weight'		: (data.fontBold == 1) ? 'bold' : 'normal',
				'font-size'			: data.fontSize + 'px',
				'font-style'		: (data.fontItalic == 1) ? 'italic' : 'normal',
				'font-family'		: '"' + data.fontFamily + '"',
				'text-decoration'	: (data.fontUnderline == 1) ? 'underline' : 'none'
			};

			this.contentElement.setStyle(style);
			if (Sb.Editor.Ui.editComponent && Sb.Editor.Ui.editComponent == this) {
				this.getEditPanel().reanchor();
			}
		}
	},

	contentFocus: function() {
		this.contentElement.focus();

		if (Ext.isIE) {
			var content = this.contentElement.dom.innerHTML;
			var position = document.selection.createRange();
			position.moveStart("textedit", content.length);
			position.collapse(false);
			position.select();
		}
	},

	initDD: function() {
		this.dd = new Ext.dd.DD(this);
		this.dd.setOuterHandleElId(this.ddEl);
		this.dd.component = this;
	}
});
Ext.reg('text_component', Sb.editor.component.Text);