/**
 * @class Sb.model.SitePage
 * @extends Sb.model.Node
 * Class for site page model
 * @constructor
 * @param {Object} config The configuration options for the site page model
 */
Sb.model.SitePage = Ext.extend(Sb.model.Node, {
	constructor: function(config) {
		this.api = {
			load: Sb.PageService.read,
			save: Sb.PageService.update
		};

		this.addEvents(
			/**
			 * @event titlevisibilitychange
			 * Fires after the title/visibility has changed.
			 */
			'titlevisibilitychange',
			/**
			 * @event credentialschange
			 * Fires after the login/password has changed.
			 */
			'credentialschange'
		);

		/**
		 * The collection of sub pages in this page as a {@link Ext.util.MixedCollection}
		 * @type MixedCollection
		 * @property pages
		 */
		this.pages = new Ext.util.MixedCollection();

		/**
		 * The model node of page widget
		 * @type Sb.model.Widget
		 * @property widget
		 */
		this.widget = new Sb.model.Widget();

		Sb.model.SitePage.superclass.constructor.call(this, config);
	},

	/**
	 * Apply function to self and to children of this page.
	 *
	 * @param {Function} fn
	 */
	applySelfAndRecursive: function(fn) {
		fn.apply(this, [this, 0, 1, 2]);
		this.applyRecursive(fn);
	},

	/**
	 * Recursively apply function to children of this page.
	 *
	 * @param {Function} fn
	 */
	applyRecursive: function(fn) {
		this.pages.each(function (item, index, length) {
			fn.apply(this, arguments);
			item.applyRecursive(fn);
		}, this);
	},

	unserialize: function(data) {
		var isChanged = false, m;

		if (undefined != data.widgetList) {
			data.rootData = {widgetList: data.widgetList};
			delete data.widgetList;
		}

		if (Ext.isArray(data.pages)) {
			this.pages.clear();
			Ext.each(data.pages, function(subpageData) {
				subpageData.rootData = data.rootData;
				var subpage = new Sb.model.SitePage(subpageData);
				if (Sb.Editor.siteView.currentPage && subpageData.identity == Sb.Editor.siteView.currentPage.data.identity) {
					Sb.Editor.siteView.currentPage = subpage;
				}
				this.pages.add(subpageData.identity, subpage);
			}, this);
			delete data.pages;
			isChanged = true;
		}

		if (Ext.isString(data.widget) && data.rootData != undefined && data.rootData.widgetList != undefined) {
			data.widget = data.rootData.widgetList[data.widget];
		}
		if (Ext.isObject(data.widget)) {
			data.widget.rootData = data.rootData;
			this.widget.unserialize(data.widget);
			delete data.widget;
			delete this.widget.data.rootData;
			isChanged = true;
		}

		if (data.fileName && (m = data.fileName.match(/^page(\d+)$/)) && (parseInt(m[1]) > Sb.model.SitePage.AUTO_ID)) {
			Sb.model.SitePage.AUTO_ID = parseInt(m[1]);
		}

		var unserializeResult = Sb.model.SitePage.superclass.unserialize.call(this, data) || isChanged;
		delete this.data.rootData;
		return unserializeResult;
	},

	restoreFromMemento: function(memento) {
		Sb.model.SitePage.superclass.restoreFromMemento.call(this, memento);
		Sb.Editor.model.rootPage.fireEvent('change', Sb.Editor.model.rootPage);
	},

	serialize: function(rootData) {
		var isInitiator	= !rootData; rootData = rootData || {};
		var data		= Sb.model.SitePage.superclass.serialize.call(this, rootData);
		var passData	= (isInitiator) ? data : rootData;

		data.widget = this.widget.serialize(passData);

		data.pages = [];
		this.pages.each(function(page) {
			data.pages.push(page.serialize(passData));
		});

		return data;
	},

	/**
	 * Returns file name of page.
	 * @return {String}
	 */
	getFileName: function() {
		return this.data.fileName + '.html';
	},

	/**
	 * Insert page.
	 * @param {Number} index The index to insert the item at.
	 * @param {Sb.model.SitePage} page Site page for inserting.
	 * @param {Sb.model.SitePage} parentPage Parent site page of inserted site page.
	 */
	insert: function(index, page, parentPage) {
		var sourceIndex = parentPage.pages.indexOf(page);
		if (sourceIndex == -1) {
			return;
		}

		parentPage.pages.remove(page);
		this.pages.insert(index, page);
		Sb.Editor.model.rootPage.fireEvent('change', Sb.Editor.model.rootPage);
	},

	/**
	 * Remove page from tree pages and fireEvent 'change' about change tree
	 * @param {Sb.model.SitePage} page
	 */
	remove: function(page) {
		this.pages.remove(page);
		Sb.Editor.model.rootPage.fireEvent('change', Sb.Editor.model.rootPage);
	},

	/**
	 * Generate uniquie identity for page instance
	 * @return {String}
	 */
	generateIdentity: function () {
		var template = '1234567890abcdefghijklmnopqrstuvwxyz';
		var rndstring = '';
		var b = 0, a;
		for (a = 0; a <= 10; a++) {
			b = Math.floor(Math.random() * template.length);
			rndstring += template.substr(b, 1);
		}
		return rndstring;
	},

	/**
	 * Create new page
	 * @param {Object} config Object for configure a page including a config widgets structure in property "widget"
	 * @return {Sb.model.SitePage} page
	 */
	createPage: function(config) {
		// NOTE: austimenko: 2010-08-27 Order is matter! See SB_Widget_Container_Abstract
		var regions				= 'content,header,column1,column2,footer'.split(',');
		var siteSharedWidgets	= {};
		var pageVerticalWidgets	= [];

		// Fill shared widgets from root page
		Ext.each(regions, function(region) {
			siteSharedWidgets[region] = [];
		});
		Sb.Editor.model.rootPage.widget.widgets.each(function(verticalContainer, index, length) {
			verticalContainer.widgets.each(function(widget, index2) {
				if (widget.isShared) {
					siteSharedWidgets[verticalContainer.data.region].push(widget.serialize());
				}
			});
		});
		// Generate vertical widgets for new page
		Ext.each(regions, function(region) {
			pageVerticalWidgets.push({
				type	: 'container_vertical',
				uuid	: Sb.Utils.generateUuid(),
				region	: region,
				widgets	: siteSharedWidgets[region]
			});
		});

		var defaultCfg	= {
			description		: Sb.Editor.model.data.description,
			fileName		: 'page' + (++Sb.model.SitePage.AUTO_ID),
			hidden			: false,
			identity		: this.generateIdentity(),
			keywords		: Sb.Editor.model.data.keywords,
			title			: SbGetLocaleByKey('newPageTitle'),
			browserTitle	: '',
			login			: null,
			password		: null,
			widget			: {
				type	: 'container_fixed',
				uuid	: Sb.Utils.generateUuid(),
				widgets	: pageVerticalWidgets
			}
		};

		return new Sb.model.SitePage(Ext.apply(defaultCfg, config || {}));
	},

	/**
	 * Removes shared widget recursively in concrete region.
	 * @param {Sb.model.ContainerVerticalWidget} sourceContainer Container, from which widget will be removed
	 * @param {Sb.model.Widget} widget Widget, that will be removed
	 */
	removeSharedWidget: function(sourceContainer, widget) {
		this.applySelfAndRecursive(function (page) {
			page.widget.widgets.each(function(verticalContainer) {
				if (sourceContainer.data.uuid == verticalContainer.data.uuid) {
					return;
				}
				if (sourceContainer.data.region != verticalContainer.data.region) {
					return;
				}
				verticalContainer.removeChild(widget, true);
			});
		});
	},

	/**
	 * Adds shared widget recursively in concrete region.
	 * @param {Sb.model.ContainerVerticalWidget} sourceContainer Container, from which widget will be removed
	 * @param {Sb.model.Widget} widget Widget, that will be added
	 * @param {Number} addIndex Index, where widget will be added
	 */
	addSharedWidget: function(sourceContainer, widget, addIndex) {
		var lastIndex	= sourceContainer.widgets.length - 1;
		var isLast		= addIndex == lastIndex;
		this.applySelfAndRecursive(function (page) {
			page.widget.widgets.each(function(verticalContainer) {
				if (sourceContainer.data.uuid == verticalContainer.data.uuid) {
					return;
				}
				if (sourceContainer.data.region != verticalContainer.data.region) {
					return;
				}
				var lastIndex	= verticalContainer.widgets.length;
				var copyIndex	= (isLast || addIndex >= lastIndex) ? lastIndex : addIndex;
				verticalContainer.copyChild(copyIndex, widget, true);
			});
		});
	},

	/**
	 * insert child page
	 * @param {Sb.model.SitePage} page
	 * @param {Number} index The index to insert the item at.
	 * @return {Sb.model.SitePage} this
	 */
	insertChild: function(page, index) {
		if (index) {
			this.pages.insert(index, page);
		} else {
			this.pages.add(page);
		}
		Sb.Editor.model.rootPage.fireEvent('change', Sb.Editor.model.rootPage);
		return this;
	},

	/**
	 * Returns self and all parents in array.
	 *
	 * @param {Sb.model.SitePage} page Non required param
	 * @return {Array}
	 */
	getSelfAndParents: function(page) {
		var isInitiator = undefined == page,
			currentPage	= isInitiator ? this : page,
			parents		= (currentPage.parentPage) ? currentPage.getSelfAndParents(currentPage.parentPage) : [];

		// Add current page
		parents.push(currentPage);

		// Remove root page
		if (isInitiator) {
			parents.shift();
		}
		return parents;
	},

	/**
	 * Gets first non-hidden page
	 *
	 * @return {Sb.model.SitePage} page
	 */
	getFirstVisiblePage: function() {
		var firstPage = null;

		this.pages.each(function(page) {
			if (!page.data.hidden) {
				firstPage = page;
				return false;
			}
		});

		return firstPage;
	},

	/**
	 * Updates level and parentPage for all pages
	 */
	updateLevelAndParent: function() {
		this.applyRecursive(function (page) {
			page.level		= (this.level || 0) + 1;
			page.parentPage	= this;
		});
	},

	/**
	 * Tells us whether we have our page protected
	 * @returns {Boolean}
	 */
	isProtected: function() {
		return (this.data.login && this.data.password) ? true : false;
	},


	/**
	 * Fires on changing of single property change to allow us fire up navigation widget change events.
	 * @param {String} key
	 */
	onUnserializePropertyChange: function (key) {
		if (key == 'title' || key == 'browserTitle' || key == 'hidden') {
			this.fireEvent('titlevisibilitychange');
		} else if (key == 'login' || key == 'password') {
			this.fireEvent('credentialschange');
		}
	}
});

Sb.model.SitePage.AUTO_ID = 1;