/**
 * @class Sb.widget.Header
 * @extends Sb.widget.Widget
 * Class for header widget
 * @constructor
 * @param {Object} config The config object
 * @xtype header_widget
 */
Sb.widget.Header = Ext.extend(Sb.widget.Widget, {

	hasEditMenu: false,

	cls: 'sb-widget-header',

	/**
	 * How many pixels of inner component's will be always visible in X axis
	 * @type Number
	 * @property componentVisibleOffsetX
	 */
	componentVisibleOffsetX: 16,

	/**
	 * How many pixels of inner component's will be always visible in Y axis
	 * @type Number
	 * @property componentVisibleOffsetY
	 */
	componentVisibleOffsetY: 16,

	initComponent: function() {
		this.addEvents('headerloaded');

		Sb.widget.Header.superclass.initComponent.apply(this, arguments);
	},

	initComponentDD: function(componentName) {
		this[componentName].dd.startDrag = function() {
			Sb.Editor.siteView.onWidgetStartDrag();
			var offsetX = this.componentVisibleOffsetX - this[componentName].el.getWidth();
			var offsetY = this.componentVisibleOffsetY - this[componentName].el.getHeight();
			this[componentName].dd.constrainTo(this.widgetContentEl, {
				left: offsetX,
				right: offsetX,
				top: offsetY,
				bottom: offsetY
			});
			if (Sb.Editor.Ui.editComponent) {
				Sb.Editor.Ui.editComponent.completeEdit.call(Sb.Editor.Ui.editComponent);
				if (!this[componentName].controlsEl.isVisible()) {
					this.disableClipping();
					this[componentName].showControls();
				}
			}
		}.createDelegate(this);
		this[componentName].dd.endDrag = function() {
			var properties = {};
			properties[componentName + 'PositionTop'] = this[componentName].getEl().getTop(true);
			properties[componentName + 'PositionLeft'] = this[componentName].getEl().getLeft(true);
			this.model.unserialize(properties);
			this.model.saveIfChanged(this.memento);
			Sb.Editor.siteView.onWidgetEndDrag();
		}.createDelegate(this);
	},

	onRender: function() {
		Sb.widget.Header.superclass.onRender.apply(this, arguments);

		this.logo = new Sb.editor.component.Image({
			cls: 'header-logo',
			renderTo: this.widgetContentEl,
			uploadAction: 'Upload/Logo',
			uploadPath: Sb.Editor.model.getUrl() + '/' + Sb.Config.attachmentsDirName + '/Logo',
			listeners: {
				scope: this,
				'remove': function() {
					this.model.unserialize({logoIsHidden: true});
					this.model.save();
				},
				'startover': this.disableClipping,
				'removeover': this.enableClipping,
				'startedit': this.disableClipping,
				'completeedit': function(logo) {
					this.enableClipping();
					var logoPath = logo.image.replace(new RegExp('^' + Sb.Editor.model.getUrl() + '(.*)'), '$1');
					var memento = this.model.saveToMemento();

					var position = this.getConstrainLeftTop(logo);
					this.model.unserialize({
						logoPath: logoPath,
						logoPositionLeft: position[0],
						logoPositionTop: position[1]
					});
					this.model.saveIfChanged(memento);
				}
			}
		});
		this.initComponentDD('logo');

		this.title = new Sb.editor.component.Text({
			id: 'header-title',
			cls: 'header-title',
			emptyText: SbGetLocaleByKey('titleEmptyMessage'),
			renderTo: this.widgetContentEl,
			listeners: {
				scope: this,
				'remove': function() {
					this.model.unserialize({titleIsHidden: true});
					this.model.save();
				},
				'startover': this.disableClipping,
				'removeover': this.enableClipping,
				'startedit': this.disableClipping,
				'completeedit': function(title) {
					this.enableClipping();
					var position = this.getConstrainLeftTop(title);
					title.data['PositionLeft'] = position[0];
					title.data['PositionTop'] = position[1];
					this.onTextComponentChange('title', title.data);
				}
			}
		});
		this.initComponentDD('title');

		this.subtitle = new Sb.editor.component.Text({
			id: 'header-subtitle',
			cls: 'header-subtitle',
			emptyText: SbGetLocaleByKey('subtitleEmptyMessage'),
			renderTo: this.widgetContentEl,
			listeners: {
				scope: this,
				'remove': function() {
					this.model.unserialize({subtitleIsHidden: true});
					this.model.save();
				},
				'startover': this.disableClipping,
				'removeover': this.enableClipping,
				'startedit': this.disableClipping,
				'completeedit': function(subtitle) {
					this.enableClipping();
					var position = this.getConstrainLeftTop(subtitle);
					subtitle.data['PositionLeft'] = position[0];
					subtitle.data['PositionTop'] = position[1];
					this.onTextComponentChange('subtitle', subtitle.data);
				}
			}
		});
		this.initComponentDD('subtitle');

		this.onImageChange();
		this.onImageFitChange();
		this.onLogoChange();
		this.onTitleChange();
		this.onSubtitleChange();
	},

	initEvents: function() {
		Sb.widget.Header.superclass.initEvents.call(this);

		this.model.on('heightchange', this.onHeightChange, this);
		this.model.on('imagechange', this.onImageChange, this);
		this.model.on('imagefitchange', this.onImageFitChange, this);
		this.model.on('logochange', this.onLogoChange, this);
		this.model.on('titlechange', this.onTitleChange, this);
		this.model.on('subtitlechange', this.onSubtitleChange, this);
		this.model.on('save', function () {
			Sb.Editor.Ui.mainWindow.statusBar.setMessage(
				SbGetLocaleByKey('messageChangesSaved')
			);
		});
		this.mon(Sb.Editor.model.design, {
			scope: this,
			save: this.onLayoutChange
		});
	},

	getConstrainLeftTop: function(c) {
		var box =  c.getEl().getBox(false, true);

		if (box.x + box.width < this.componentVisibleOffsetX) {
			box.x = this.componentVisibleOffsetX - box.width;
		}

		if (box.y + box.height < this.componentVisibleOffsetY) {
			box.y = this.componentVisibleOffsetY - box.height;
		}
		return [box.x, box.y];
	},

	onHeightChange: function() {
		var data			= this.model.data,
			contentHeight	= data.imageName == 'fallback' ? data.height : 'auto',
			bordersHeight	= 2,
			isNeedReanchor	= Sb.Editor.Ui.editComponent == this;

			this.widgetContentEl.setHeight(contentHeight);

			var isNeedVAlign = this.widgetContentEl.getHeight() > bordersHeight;
			if (isNeedVAlign) {
				this.alignVerticalContent('logo');
				this.alignVerticalContent('title');
				this.alignVerticalContent('subtitle');
			}

			if (isNeedReanchor) {
				Sb.widget.Header.editPanel.reanchor();
			}
	},

	onLayoutChange: function() {
		var data = this.model.data;
		this.alignHorizontalContent('logo');
		this.alignHorizontalContent('title');
		this.alignHorizontalContent('subtitle');
		this.model.saveIfChanged(this.memento);
	},

	/**
	 * Vertical align components like title when header resizes
	 *
	 * @param {String} componentName For example 'title'
	 **/
	alignVerticalContent: function(componentName) {
		var data			= this.model.data,
			headerHeight	= parseInt(this.widgetContentEl.getHeight()),
			dataKey			= componentName + 'PositionTop',
			oldTop			= parseInt(data[dataKey]),
			bottom			= this.componentVisibleOffsetY + oldTop,
			isNeedAlign		= bottom > headerHeight,
			newTop			= Math.max(headerHeight - this.componentVisibleOffsetY, 0);

		if (isNeedAlign) {
			this.model.setProperty(dataKey, newTop);
		}
	},

	/**
	 * Horizontal align components like title when header resizes
	 *
	 * @param {String} componentName For example 'title'
	 **/
	alignHorizontalContent: function(componentName) {
		var headerWidth = parseInt(this.widgetContentEl.getWidth());
		var oldLeft = parseInt(this.model.data[componentName + 'PositionLeft']);
		var right = this.componentVisibleOffsetX + oldLeft;
		if (right > headerWidth) {
			var newLeft = headerWidth - this.componentVisibleOffsetX;
			newLeft = (newLeft < 0) ? 0 : newLeft;
			var newData = {};
			newData[componentName + 'PositionLeft'] = newLeft;
			this.model.unserialize(newData);
		}
	},

	onImageChange: function() {
		var data = this.model.data;
		var path = '' == data.imagePath
			? Sb.Config.headersUrl + '/' + data.imageName + '/header.png'
			: Sb.Editor.model.getUrl() + data.imagePath;

		//add timestamp for custom headers to avoid caching
		if (data.imageTime) {
			path += '?' + data.imageTime;
		}

		// destroy image and create new for change image (because of 'onload' bug in ie)
		if (this.imageEl) {
			this.imageEl.un('load');
			Ext.destroy(this.imageEl);
		}
		this.imageEl = this.widgetContentEl.createChild(
			{tag: 'img', cls: 'header-image'}, this.headerContentEl
		);
		if (Sb.widget.Header.imageSize) {
			this.imageEl.setSize(Sb.widget.Header.imageSize);
		}

		this.imageEl.on('load', this.onImageLoad, this, {single: true});
		this.imageEl.set({src: path});
	},

	onImageLoad: function() {
		var data	= this.model.data,
			top		= data.imagePositionTop + 'px',
			left	= data.imagePositionLeft + 'px';

		this.imageEl.setStyle({top: top, left: left});
		this.onImageFitChange();
		this.fireEvent('headerloaded', this);
		this.onHeightChange();
	},

	onImageFitChange: function() {
		var data			= this.model.data,
			isFit			= 1 == data.imageFit,
			isNeedReachor	= Sb.Editor.Ui.editComponent == this;

		this.imageEl.setSize('auto', 'auto');
		Sb.widget.Header.imageSize = this.imageEl.getSize();
		if (isFit) {
			this.imageEl.setStyle({width: '100%'});
		}

		if (isNeedReachor) {
			Sb.widget.Header.editPanel.reanchor();
		}
	},

	onLogoChange: function() {
		var data		= this.model.data,
			defaultPath	= Sb.Config.logosUrl + '/' + data.logoName + '/logo.png',
			customPath	= Sb.Editor.model.getUrl() + data.logoPath,
			path		= '' == data.logoPath ? defaultPath : customPath,
			isVisible	= 1 != data.logoIsHidden;

		this.logo.setImage(path);
		this.logo.setVisible(isVisible);
		this.logo.el.setLeftTop(data.logoPositionLeft, data.logoPositionTop);
	},

	onTitleChange: function() {
		var data		= this.model.data,
			isVisible	= 1 != data.titleIsHidden;

		this.title.setData({
			content			: data.title,
			fontColor		: data.titleFontColor,
			fontBold		: data.titleFontBold,
			fontSize		: data.titleFontSize,
			fontItalic		: data.titleFontItalic,
			fontFamily		: data.titleFontFamily,
			fontUnderline	: data.titleFontUnderline
		});
		this.title.setVisible(isVisible);
		this.title.el.setLeftTop(data.titlePositionLeft, data.titlePositionTop);
	},

	onSubtitleChange: function() {
		var data = this.model.data;
		this.subtitle.setData({
			content: data.subtitle,
			fontColor: data.subtitleFontColor,
			fontBold: data.subtitleFontBold,
			fontSize: data.subtitleFontSize,
			fontItalic: data.subtitleFontItalic,
			fontFamily: data.subtitleFontFamily,
			fontUnderline: data.subtitleFontUnderline
		});
		this.subtitle.setVisible(data.subtitleIsHidden != 1);
		this.subtitle.el.setLeftTop(data.subtitlePositionLeft, data.subtitlePositionTop);
	},

	onCompleteEdit: function() {
		this.model.saveIfChanged(this.memento);
	},

	onDestroy: function() {
		if (this.rendered) {
			this.model.un('heightchange', this.onHeightChange, this);
			this.model.un('imagechange', this.onImageChange, this);
			this.model.un('imagefitchange', this.onImageFitChange, this);
			this.model.un('logochange', this.onLogoChange, this);
			this.model.un('titlechange', this.onTitleChange, this);
			this.model.un('subtitlechange', this.onSubtitleChange, this);
		}
		Sb.widget.Header.superclass.onDestroy.call(this);
	},

	onDocumentClick: function(event) {
		if (event.getTarget('.sb-color-dropdown')
			|| event.getTarget('.sb-widget-edit-panel-header')
			|| event.getTarget('.x-combo-list')
		) {
			return;
		}

		Sb.widget.Header.superclass.onDocumentClick.call(this, event);
	},

	onTextComponentChange: function(componentName, data) {
		data = data || {};
		var modelData = {};
		Ext.iterate(data, function(property, value) {
			if (property == 'content') {
				modelData[componentName] = value;
			} else {
				modelData[componentName + property.toUpperCaseFirst()] = value;
			}
		}, this);

		this.model.unserialize(modelData);
		this[componentName].getEditPanel().reanchor();

		this.model.saveIfChanged(this.memento);
	},

	enableClipping: function() {
		this.widgetEl.setStyle({position: 'relative'});
	},

	disableClipping: function() {
		this.widgetEl.setStyle({position: 'static'});
		this.manager.bringToFront(this);
	}
});

Ext.reg('header_widget', Sb.widget.Header);