/**
 * @class Sb.widget.Navigation
 * @extends Sb.widget.Widget
 * Class for Navigation widget
 * @constructor
 * @param {Object} config The config object
 * @xtype navigation_widget
 */
Sb.widget.Navigation = Ext.extend(Sb.widget.Widget, {

	cls: 'sb-widget-navigation',

	/**
	 * @cfg {Sb.model.NavigationWidget} model
	 * The {@link Sb.model.NavigationWidget} to bind this Widget to.
	 */

	onRender: function() {
		Sb.widget.Navigation.superclass.onRender.apply(this, arguments);

		this.innerEl.addClass('clearfix');

		this.menu.add({
			text: SbGetLocaleByKey('cmPages'),
			iconCls: 'sb-pages-icon',
			scope: this,
			handler: function() {
				this.completeEdit();
				Sb.Editor.Ui.mainWindow.tabs.setActiveTab('pages');
			}
		});
		this.menu.add({
			text: SbGetLocaleByKey('cmWidgetNavigationItems'),
			iconCls: 'sb-menu-items-style-icon',
			scope: this,
			handler: function() {
				this.startEdit();
				Sb.widget.Navigation.editPanel.tabs.setActiveTab(1); // Items panel
			}
		});
	},

	afterRender: function() {
		Sb.widget.Navigation.superclass.afterRender.call(this);
		this.refresh();
		this.onFontChange();
		this.onColorChange();
		this.onAlignmentChange();
	},

	initEvents: function() {
		Sb.widget.Navigation.superclass.initEvents.call(this);

		//		Observable					Event				Handler					Scope
		this.mon(Sb.Editor.model.rootPage,	'change',			this.refresh,			this);
		this.mon(this.model,				'levelchange',		this.refresh,			this);
		this.mon(this.model,				'fontchange',		this.onFontChange,		this);
		this.mon(this.model,				'colorchange',		this.onColorChange,		this);
		this.mon(this.model,				'alignmentchange',	this.onAlignmentChange,	this);
		this.mon(this.model,				'save',				function () {
			Sb.Editor.Ui.mainWindow.statusBar.setMessage(
				SbGetLocaleByKey('messageChangesSaved')
			);
		});

		if (Ext.isIE7) {
			this.getEl().on('mouseenter', function() {
				var parentNode = Ext.get(this.el.findParentNode('#header-content'));
				if (!Sb.Editor.Ui.isDragActive() && !Sb.Editor.Ui.editComponent && parentNode) {
					parentNode.setStyle('z-index', this.el.getStyle('z-index'));
				}
			}, this);
			this.getEl().on('mouseleave', function() {
				var parentNode = Ext.get(this.el.findParentNode('#header-content'));
				if (!Sb.Editor.Ui.isDragActive() && !Sb.Editor.Ui.editComponent && parentNode) {
					parentNode.setStyle('z-index', 0);
				}
			}, this);
		}
	},

	/**
	 * Remove "over" state
	 */
	removeOver: function() {
		Sb.widget.Navigation.superclass.removeOver.call(this);
		if (!Sb.Editor.Ui.isDragActive() && Sb.Editor.Ui.editComponent != this) {
			var parentNode = Ext.get(this.el.findParentNode('#header-content'));
			if (parentNode) {
				Ext.select('#columns .sb-widget-navigation').each(function () {
					var item = Ext.getCmp(this.id);
					item.manager.sendToBack(item);
				});
			}
		}
	},

	/**
	 * Gets parent page for which menu is generated.
	 *
	 * @param {Sb.model.SitePage} page Can be null, when called 1st time
	 * @return {Sb.model.SitePage}
	 */
	getParentPage: function(page) {
		var page = page || Sb.Editor.siteView.currentPage,
			data = this.model.data;

		if (data.level == page.level) {
			page = page.parentPage;
		} else if (data.level < page.level) {
			page = this.getParentPage(page.parentPage);
		} else if ((data.level - page.level) > 1) {
			page = null;
		}

		return page;
	},

	/**
	 * Create menu item and create all items of submenu
	 * @param {dom element} parentPage
	 * @param {Sb.model.SitePage} page
	 * @param {Number} level
	 */
	createMenuItem: function(parentEl, page, level) {
		level = level || 1;
		var selectedCls = '';
		if (this.isSelectedSubpage(page)) {
			selectedCls = 'childselected';
		}
		if (Sb.Editor.siteView.currentPage.data.identity == page.data.identity) {
			selectedCls = 'selected';
		}

		var itemEl = parentEl.createChild({
			tag : 'li',
			cls :  selectedCls,
			cn  : {
				tag  : 'a',
				href : '#' + escape(page.getFileName()),
				cn: [{
					tag	: 'span',
					html: '>',
					cls	: 'navigation-item-bullet'
				}, {
					tag	: 'span',
					cls	: 'navigation-item-text',
					html: Ext.util.Format.htmlEncode(page.data.title)
				}]
			}
		});
		itemEl.setVisibilityMode(Ext.Element.DISPLAY);
		itemEl.setVisible(!page.data.hidden);
		this.addInternalListener(itemEl, 'click', function(event, el, options) {
			event.stopEvent();
			Sb.Editor.siteView.setCurrentPage(options.page);
		}, null, {delegate: 'a', 'page' : page});
		this.addInternalListener(page, 'titlevisibilitychange', function() {
			itemEl.child('.navigation-item-text').update(Ext.util.Format.htmlEncode(page.data.title));
			itemEl.setVisible(!page.data.hidden);
		});

		if (this.model.data.depth > level && page.pages.getCount()) {
			itemEl.addClass('navigation-item-expand');
			var submenuEl = itemEl.createChild({
				tag : 'ul'
			});
			page.pages.each(function(subPage) {
				this.createMenuItem(submenuEl, subPage, level + 1);
			}, this);
		}
		return itemEl;
	},

	isSelectedSubpage: function(page) {
		page.pages.each(function(subPage) {
			if (Sb.Editor.siteView.currentPage.data.identity == subPage.data.identity) {
				this.isSelected = true;
				return false;
			}
			this.isSelected = this.isSelectedSubpage(subPage);
		}, this);
		var isSelected = this.isSelected;
		this.isSelected = false;
		return isSelected;
	},

	/**
	 * Is widget has content, that could be rendered? Used in renderPlaceholder, for instance.
	 *
	 * @returns {Boolean}
	 */
	isEmpty: function() {
		var menuPages	= this.getParentPage(),
			isEmpty		= true;

		if (menuPages == null) {
			return isEmpty;
		}
		menuPages.pages.each(function(page) {
			if (!page.data.hidden) {
				isEmpty = false;
				return false;
			}
		}, this);

		return isEmpty;
	},

	/**
	 * Render content for widget`s placeholder if it has one
	 * @return {Sb.widget.Widget} self
	 */
	renderPlaceholder: function() {
		Sb.widget.Navigation.superclass.renderPlaceholder.apply(this, arguments);
		this.togglePlaceholder(true);
		this.update(this.placeholderTpl.apply({
			placeholderMsg: SbGetLocaleByKey('navigationWidgetActiveText')
		}));

		// When we have all pages of current parent page hidden, we should rerender widget on visibility change
		var parentPage = this.getParentPage();
		if (parentPage) {
			parentPage.pages.each(function(page) {
				this.addInternalListener(page, 'titlevisibilitychange', function() {
					this.refresh();
				});
			}, this);
		}
	},

	refresh: function() {
		this.cleanInternalListeners();

		if (this.isEmpty()) {
			this.renderPlaceholder();
			return;
		}

		this.togglePlaceholder(false);
		this.update('');
		this.navigationEl = this.widgetContentEl.createChild({
			tag: 'ul',
			id:  'navigation-' + this.model.data.uuid,
			cls: 'navigation'
		});
		this.widgetContentEl.createChild({
			cls: 'cleared',
			html: '<!-- clear -->'
		});

		this.getParentPage().pages.each(function(page) {
			var itemEl = this.createMenuItem(this.navigationEl, page);
			if (this.isSelectedSubpage(page)) {
				itemEl.addClass('childselected');
			}
			// When we have all pages of current parent page hidden, we should rerender widget on visibility change
			this.addInternalListener(page, 'titlevisibilitychange', function() {
				if (this.isEmpty()) {
					this.refresh();
				}
			});

		}, this);

		if (Sb.Editor.Ui.editComponent == this) {
			this.getEditPanel().reanchor();
		}
	},

	onCompleteEdit: function() {
		this.model.saveIfChanged(this.memento);
		if (Ext.isIE7) {
			var parentNode = Ext.get(this.el.findParentNode('#header-content'));
			if (parentNode) {
				parentNode.setStyle('z-index', 0);
			}
		}
	},

	onDocumentClick: function(event) {
		if (event.getTarget('.x-combo-list') || event.getTarget('.sb-color-dropdown')) {
			return;
		}

		Sb.widget.Navigation.superclass.onDocumentClick.call(this, event);
	},

	/**
	 * Updates CSS for navigation widget fonts.
	 */
	onFontChange: function() {
		var id		= 'navigation-' + this.model.data.uuid;
		var data	= Ext.apply({className: '#' + id}, this.model.data);
		Ext.iterate(data, function (key, value, hash) {
			var intValue = parseInt(value);
			if (Ext.isNumber(intValue)) {
				hash[key] = intValue;
			}
		});
		data['level1LineSize']	= data['level1FontSize'] + 10;
		data['level2LineSize']	= data['level2FontSize'] + 10;
		Sb.StyleManager.addStyleSheet(
			this.getType().getCssTemplate().apply(data),
			'widget-navigation',
			'font-' + this.model.data.uuid
		);
		if (Sb.Editor.Ui.editComponent == this) {
			this.getEditPanel().reanchor();
		}
	},

	/**
	 * Updates CSS for navigation widget colors.
	 */
	onColorChange: function() {
		var id		= 'navigation-' + this.model.data.uuid;
		var data	= Ext.apply({className: '#' + id}, this.model.data);
		Sb.StyleManager.addStyleSheet(
			this.getType().getColorSchemeCssTemplate().apply(data),
			'widget-navigation',
			'color-' + this.model.data.uuid
		);
	},

	/**
	 * Updates CSS for navigation widget alignment.
	 */
	onAlignmentChange: function() {
		if (this.navigationEl && this.model.data.level1Alignment) {
			this.navigationEl.setStyle('text-align', this.model.data.level1Alignment.toLowerCase());
		}
	}
});

Ext.reg('navigation_widget', Sb.widget.Navigation);