/**
 * @class Sb.widget.Widget
 * @extends Sb.editor.component.Base
 * Base class for widgets
 * @constructor
 * @param {Object} config The config object
 */
Sb.widget.Widget = Ext.extend(Sb.editor.component.Base, {
	/**
	 * @cfg {Sb.model.Widget} model
	 * The {@link Sb.model.Widget} to bind this Widget to.
	 */

	hasEditMenu: true,

	/**
	 * The template to use when rendering the Widget.
	 *
	 * @property tpl
	 * @type Ext.Template
	 */
	tpl: new Ext.Template(
		'<div id="widget-{uuid}" class="widget widget-{type}">',
			'<div class="widget-content">{content}</div>',
		'</div>',
		{compiled: true}
	),

	/**
	 * The template to use when rendering the placeholder of widget.
	 *
	 * @property placeholderTpl
	 * @type Ext.Template
	 */
	placeholderTpl: new Ext.Template(
		'<table cellpadding="0" cellspacing="0">',
			'<tr>',
				'<td><img src="images/blank.gif" align="absmiddle"/></td>',
				'<td>{placeholderMsg}</td>',
			'</tr>',
		'</table>',
		{compiled: true}
	),

	initComponent: function() {
		Sb.widget.Widget.superclass.initComponent.apply(this, arguments);

		/**
		 * Internal listeners that could be cleaned up when we need in some intermediate stage of component lifetime
		 *
		 * @type Array
		 * @property internalListeners
		 *
		 */
		this.internalListeners = [];
	},

	/**
	 * Is widget has content, that could be rendered? Used in renderPlaceholder, for instance.
	 *
	 * @returns {Boolean}
	 */
	isEmpty: function() {
		return false;
	},

	onRender: function() {
		Sb.widget.Widget.superclass.onRender.apply(this, arguments);

		this.tpl.overwrite(this.contentElement, {
			uuid: this.model.data.uuid,
			type: this.model.data.type,
			content: this.renderContent()
		});

		this.widgetEl = this.contentElement.child('div.widget-' + this.model.data.type);
		this.widgetContentEl = this.contentElement.child('div.widget-content');
	},

	/**
	 * Render content for widget
	 * @return {String} Rendered content
	 */
	renderContent: function() {
		return '';
	},

	/**
	 * Render content for widget`s placeholder if it has one
	 * @return {Sb.widget.Widget} self
	 */
	renderPlaceholder: function() {
		return this;
	},

	getContentTarget: function() {
		return this.widgetContentEl;
	},

	/**
	 * Returns the <code>id</code> of this widget (sb-widget-UUID)
	 * @return {String} id
	 */
	getId: function() {
		return this.id || (this.id = 'sb-widget-' + this.model.data.uuid);
	},

	/**
	 * Returns the <code>type</code> of this widget`s model
	 *
	 * @return {Sb.widgetType.Base}
	 */
	getType: function() {
		return Sb.WidgetTypeManager.get(this.model.data.type);
	},

	/**
	 * Tries to get edit panel for current type or it`s parent.
	 *
	 * @param {xtype} type
	 * @return {Sb.editor.component.BaseEditPanel} editPanel
	 */
	getEditPanel: function(type) {
		if (undefined == type) {
			type = Ext.ComponentMgr.types[Sb.Editor.Ui.editComponent.getXType()];
		}
		if (type.editPanel) {
			return type.editPanel;
		} else if (type.superclass) {
			return this.getEditPanel(type.superclass.constructor);
		}
		return undefined;
	},

	getStateCls: function(state) {
		var cls = this.baseCls;
		if (this.model.isShared) {
			cls += '-shared';
		}
		cls += '-' + state;
		return cls;
	},

	afterRender: function() {
		Sb.widget.Widget.superclass.afterRender.call(this);

		if (this.model.isNew) {
			this.model.isNew = false;
			this.el.scrollIntoView(Sb.Editor.siteView.container);
			this.startEdit();
		}
	},

	removeComponentHandler: function() {
		Sb.widget.Widget.superclass.removeComponentHandler.call(this);
		this.ownerCt.model.removeChild(this.model);
	},

	onStartEdit: function() {
		this.memento = this.model.saveToMemento();
		this.getType().getEditPanel().show(this);
	},

	cancelEdit: function() {
		if (Sb.Editor.Ui.editComponent != this) {
			return;
		}
		if (this.memento) {
			this.model.restoreFromMemento(this.memento);
		}
		this.onCancelEdit();
		this.hideEdit();
	},

	onHideEdit: function() {
		this.getType().getEditPanel().hide();
	},

	/**
	 * Add internal listener
	 *
	 * @param {Object} item
	 * @param {String} ename
	 * @param {Function} fn
	 * @param {Object} scope
	 * @param {Object} opt
	 */
	addInternalListener: function(item, ename, fn, scope, opt) {
		scope	= scope || this;
		opt		= opt || {};
		this.mon(item, ename, fn, scope, opt);
		this.internalListeners.push({item: item, ename: ename, fn: fn, scope: scope, opt: opt});
	},

	/**
	 * Cleans up internal listeners and unsubscribes all of them
	 */
	cleanInternalListeners: function() {
		Ext.each(this.internalListeners, function(internalListener) {
			this.mun(internalListener.item, internalListener.ename, internalListener.fn, internalListener.scope);
		}, this);
	},

	onDestroy: function() {
		Sb.widget.Widget.superclass.onDestroy.call(this);
		if (this.rendered) {
			this.model.destroy();
		}
		if (Sb.Editor.Ui.overComponent == this) {
			if (Sb.Editor.Ui.overComponent.showOverTask) {
				Sb.Editor.Ui.overComponent.showOverTask.cancel();
			}
			Sb.Editor.Ui.overComponent = null;
		}
		if (Sb.Editor.Ui.editComponent == this) {
			Sb.Editor.Ui.editComponent = null;
		}
	},

	initDD: function() {
		this.dd = new Sb.widget.DD(this);
	},

	/**
	 * Enables/disables placeholder mode by adding/removing class to main element
	 *
	 * @param {Boolean} isEnable
	 */
	togglePlaceholder: function(isEnable) {
		this.el[(isEnable ? 'add' : 'remove' ) + 'Class']('sb-widget-placeholder');
	}
});

//private - custom Widget DD implementation
Sb.widget.DD = function(widget, cfg) {
	this.component = widget;
	cfg = Ext.apply({ddGroup: 'widgets'}, cfg);

	Sb.widget.DD.superclass.constructor.call(this, this.component.el, cfg);
	this.setOuterHandleElId(this.component.ddEl);
	this.component.ddEl.setStyle('cursor', 'move');
	this.scroll = false;
};

Ext.extend(Sb.widget.DD, Ext.dd.DragSource, {
	getDragEl: function() {
		return Ext.getDom(this.component.getEl());
	},

	b4MouseDown: function(e) {
		var x = e.getPageX();
		var y = e.getPageY();
		this.autoOffset(x, y);
	},

	autoOffset: function(x, y) {
		x -= this.startPageX;
		y -= this.startPageY;
		this.setDelta(x, y);
	},

	startDrag: function (x, y) {
		Sb.Editor.siteView.onWidgetStartDrag();

		if (Sb.Editor.Ui.editComponent) {
			Sb.Editor.Ui.editComponent.completeEdit.call(Sb.Editor.Ui.editComponent);
			if (!this.component.controlsEl.isVisible()) {
				this.component.showControls();
			}
		}

		// :TODO: echernodarov 20100420: dirty hack, need rework
		this.component.afterDD = true;
		this.component.setZIndex(Sb.Editor.windowManager.dragZIndex);
		this.component.ownerCt.notifyOver(this, [x, y]);
		this.component.el.setStyle({
			position: 'absolute',
			width: this.component.el.getWidth() + 'px'
		});
		if (Ext.isIE7) {

			/**
			 * Widget grand-parent container during DD
			 *
			 * @type Ext.Element
			 * @property ddWidgetContainer
			 */
			this.ddWidgetContainer = this.getWidgetContainer();
			if (this.ddWidgetContainer) {
				this.ddWidgetContainer.setStyle('z-index', Sb.Editor.windowManager.dragZIndex);
			}
		}
	},

	getWidgetContainer: function() {
		return this.component.el.up('.container-content-inner', 1).up('div', 1);
	},

	endDrag: function(e) {
		if (Ext.isIE7) {
			if (this.ddWidgetContainer) {
				this.ddWidgetContainer.setStyle('z-index', 0);
			}
			this.ddWidgetContainer = null;
		}
		delete this.invalidDrop;
		Sb.Editor.siteView.onWidgetEndDrag();
	},

	onInvalidDrop: function () {
		this.invalidDrop = true;
		var animCfgObj = {
			easing		: 'elasticOut',
			duration	: 0.3,
			scope		: this,
			callback	: function() {
				Sb.Editor.siteView.ddSpacer.remove();
				this.component.resetStyles();
			}
		};

		this.el.moveTo(this.startPageX, this.startPageY, animCfgObj);
	},
	createFrame	: Ext.emptyFn,
	showFrame	: Ext.emptyFn
});