/**
 * @class Sb.widgetType.Shared
 * @extends Sb.widgetType.Base
 */
Sb.widgetType.Shared = Ext.extend(Sb.widgetType.Base, {

	/**
	 * Source of CSS template that will be compiled
	 *
	 * @type Array
	 * @property cssTemplateSource
	 */
	cssTemplateSource: [],

	/**
	 * Source of color scheme CSS template that will be compiled
	 *
	 * @type Array
	 * @property colorSchemeCssTemplateSource
	 */
	colorSchemeCssTemplateSource: [],

	/**
	 * Source of font color CSS template that will be compiled
	 *
	 * @type Array
	 * @property fontColorCssTemplateSource
	 */
	fontColorCssTemplateSource: [],

	constructor: function(name) {
		/**
		 * Cache of widgets by uuid
		 *
		 * @type Ext.util.MixedCollection
		 * @property cache
		 */
		this.cache = new Ext.util.MixedCollection(false,
			/**
			 * Provide key for collection
			 * @param {Sb.model.Widget} widget
			 * @return {String}
			 */
			function(widget) {
				return widget.data.uuid;
			}
		);

		this.cache.on('add', this.onCacheChange, this);
		this.cache.on('remove', this.onCacheChange, this);

		/**
		 * CSS template of shared widget
		 *
		 * @type Ext.XTemplate
		 * @property cssTemplate
		 */
		this.cssTemplate = null;

		/**
		 * Color scheme CSS template of shared widget
		 *
		 * @type Ext.XTemplate
		 * @property colorSchemeCssTemplate
		 */
		this.colorSchemeCssTemplate = null;

		/**
		 * Font color CSS template of shared widget
		 *
		 * @type Ext.XTemplate
		 * @property fontColorCssTemplate
		 */
		this.fontColorCssTemplate = null;

		Sb.widgetType.Shared.superclass.constructor.call(this, name);
	},

	/**
	 * Gets type for creating child
	 * @return {String}
	 */
	getPhpType: function() {
		var reShared = /^shared/;
		if (reShared.test(this.name)) {
			return 'shared_' + this.name.replace(reShared, '').toLowerCase();
		}
		return this.name;
	},

	/**
	 * Fires when number of shared widgets in site changed
	 */
	onCacheChange: Ext.emptyFn,

	/**
	 * Instance factory with cache by uuid
	 *
	 * @param {Object} config The configuration options for the widget model
	 * @return {Sb.model.Widget}
	 */
	createInstance: function(config) {
		if (config.uuid && !this.cache.map[config.uuid]) {
			var widget = Sb.widgetType.Shared.superclass.createInstance.call(this, config);
			if (widget.isShared) {
				this.cache.add(widget);
			} else {
				return widget;
			}
		}
		return this.cache.map[config.uuid];
	},

	/**
	 * Widget cleaner by type
	 *
	 * @param {Sb.model.Widget} widget
	 */
	removeInstance: function(widget) {
		if (widget.isShared) {
			this.cache.remove(widget);
		}
	},

	/**
	 * Restore widget in cache
	 *
	 * @param {Sb.model.Widget} widget
	 */
	restoreInstance: function(widget) {
		if (widget.isShared && widget.data.uuid && !this.cache.map[widget.data.uuid]) {
			this.cache.add(widget);
		}
	},


	/**
	 * Gets CSS template of shared widget by key.
	 * @return {Ext.XTemplate}
	 */
	getCssTemplateByKey: function(key) {
		var templateKey = (key + 'CssTemplate').toLowerCaseFirst(),
			sourceKey	= (key + 'CssTemplateSource').toLowerCaseFirst();

		if (null == this[templateKey]) {
			this[templateKey] = new Ext.XTemplate(this[sourceKey]);
			this[templateKey].compile();
		}
		return this[templateKey];
	},

	/**
	 * Gets CSS template of shared widget.
	 * @return {Ext.XTemplate}
	 */
	getCssTemplate: function() {
		return this.getCssTemplateByKey('');
	},

	/**
	 * Gets color scheme CSS template of shared widget.
	 * @return {Ext.XTemplate}
	 */
	getColorSchemeCssTemplate: function() {
		return this.getCssTemplateByKey('colorScheme');
	},

	/**
	 * Gets font color CSS template of shared widget.
	 * @return {Ext.XTemplate}
	 */
	getFontColorCssTemplate: function() {
		return this.getCssTemplateByKey('fontColor');
	}
});